@@ -1,7 +1,7 @@
 /* Main command processing loop.
 
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2022 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2023 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -412,6 +412,61 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		move_inc_down(b);
 		return OK;
 
+	case BRACKETEDPASTE_A:
+		if (!bracketed_paste) {
+			print_message("Bracketed Paste support is globally disabled.");
+			free(p);
+			return OK;
+		}
+		if (!p || (p && p[0] == '?')) {
+			free(p);
+			snprintf(msg, MAX_MESSAGE_SIZE, "BracketedPaste: 0|1|macro_before macro_after");
+			p = request_string(b, msg, cur_bracketed_paste_value(b), true, COMPLETE_NONE, b->encoding == ENC_UTF8 || b->encoding == ENC_ASCII && b->opt.utf8auto);
+		}
+		if (p) {
+			if ((*p == '0' || *p == '1') && *(p+1) == '\0') {
+				c = *p - '0';
+				if (b->bpaste_support == c) {
+					free(p);
+					return OK;
+				}
+				b->bpaste_support = c;
+				if (c == 0)	turn_off_bracketed_paste();
+				if (c == 1) turn_on_bracketed_paste();
+				free(b->bpaste_macro_before);
+				free(b->bpaste_macro_after);
+				b->bpaste_macro_before = NULL;
+				b->bpaste_macro_after = NULL;
+			}
+			else {
+				char *q = p;
+				char *m1, *m2;
+				while (*q && *q == ' ') q++; /* skip leading spaces */
+				m1 = q;                      /* start of 1st macro name */
+				while (*q && *q != ' ') q++; /* keep non-spaces */
+				if (*q == ' ') {
+					*q = '\0';                /* mark end of 1st macro name */
+					q++;
+				}
+				while (*q && *q == ' ') q++; /* skip middle spaces */
+				m2 = q;                      /* start of 2nd macro name */
+				while (*q && *q != ' ') q++; /* keep non-spaces */
+				*q = '\0';                   /* mark end of 2nd macro name */
+				if (strlen(m1) && strlen(m2)) {
+					free(b->bpaste_macro_before);
+					b->bpaste_macro_before = str_dup(m1);
+					free(b->bpaste_macro_after);
+					b->bpaste_macro_after = str_dup(m2);
+					b->bpaste_support = 2;
+				} else {
+					free(p);
+					return INVALID_BRACKETED_PASTE_DESIGNATION;
+				}
+			}
+			free(p);
+		}
+		return OK;
+
 	case UNSETBOOKMARK_A:
 		if (p && p[0] == '*' && !p[1]) { /* Special parm "*" for UNSETBOOKMARK_A */
 			b->bookmark_mask = b->cur_bookmark = 0;
@@ -1043,6 +1098,11 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		ttysize();
 		keep_cursor_on_screen(cur_buffer);
 		reset_window();
+		if (executing_macro) {
+			refresh_window(b);
+			draw_status_bar();
+		}
+		cur_buffer->act = ++buffer_actuations;
 		return OK;
 
 	case FIND_A:
@@ -1473,14 +1533,11 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		return OK;
 
 	case PLAY_A:
-		if (!recording_macro && !executing_macro) {
-			if (c < 0 && (c = request_number(b, "Times", 1))<=0) return NUMERIC_ERROR(c);
-			executing_macro = 1;
-			for(int64_t i = 0; i < c && !(error = play_macro(b->cur_macro)); i++);
-			executing_macro = 0;
-			return print_error(error) ? ERROR : 0;
-		}
-		else return ERROR;
+		if (c < 0 && (c = request_number(b, "Times", 1))<=0) return NUMERIC_ERROR(c);
+		if (recording_macro) add_to_stream(recording_macro, "# include macro <unnamed>", 26);
+		error = play_macro(b->cur_macro, c);
+		if (recording_macro) add_to_stream(recording_macro, "# conclude macro <unnamed>", 27);
+		return print_error(error) ? ERROR : 0;
 
 	case SAVEMACRO_A:
 		if (p || (p = request_file(b, "Macro Name", NULL))) {
@@ -1526,6 +1583,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 			unset_interactive_mode();
 			exit(0);
 		}
+		cur_buffer->act = ++buffer_actuations;
 		keep_cursor_on_screen(cur_buffer);
 		reset_window();
 
@@ -1538,6 +1596,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 	case NEXTDOC_A: /* Was NEXT_BUFFER: */
 		if (b->b_node.next->next) cur_buffer = (buffer *)b->b_node.next;
 		else cur_buffer = (buffer *)buffers.head;
+		cur_buffer->act = ++buffer_actuations;
 		keep_cursor_on_screen(cur_buffer);
 		reset_window();
 		need_attr_update = false;
@@ -1547,6 +1606,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 	case PREVDOC_A:
 		if (b->b_node.prev->prev) cur_buffer = (buffer *)b->b_node.prev;
 		else cur_buffer = (buffer *)buffers.tail_pred;
+		cur_buffer->act = ++buffer_actuations;
 		keep_cursor_on_screen(cur_buffer);
 		reset_window();
 		need_attr_update = false;
@@ -1557,6 +1617,7 @@ int do_action(buffer *b, action a, int64_t c, char *p) {
 		const int n = request_document();
 		if (n < 0 || !(b = get_nth_buffer(n))) return ERROR;
 		cur_buffer = b;
+		cur_buffer->act = ++buffer_actuations;
 		keep_cursor_on_screen(cur_buffer);
 		reset_window();
 		need_attr_update = false;
