@@ -1,7 +1,7 @@
 /* Input line handling.
 
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2022 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2023 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -39,10 +39,10 @@
    buffer, but this could change in the future.
 
    completion_type has several possible values:
-    0 COMPLETE_NONE   means no completion,
-    1 COMPLETE_FILE   complete as a filename,
-    2                 complete as a command followed by a filename, (not implemented?)
-    3 COMPLETE_SYNTAX complete as a recognized syntax name.
+    0 COMPLETE_NONE     means no completion,
+    1 COMPLETE_FILE     complete as a filename,
+    2 COMPLETE_CMD      complete as a command followed by a filename, (not implemented?)
+    3 COMPLETE_SYNTAX   complete as a recognized syntax name.
 
    If prefer_utf8 is true, editing an ASCII line inserting an ISO-8859-1 character
    will turn it into an UTF-8 line.
@@ -614,7 +614,7 @@ static char *complete_filename(const char *start_prefix) {
 		for(struct dirent * de; !stop && (de = readdir(d)); ) {
 			if (is_prefix(start_prefix, de->d_name))
 				if (cur_prefix) {
-					cur_prefix[max_prefix(cur_prefix, de->d_name)] = 0;
+					cur_prefix[max_prefix(cur_prefix, ENC_8_BIT, de->d_name, ENC_8_BIT)] = 0;
 					unique = false;
 				}
 				else {
@@ -745,8 +745,10 @@ char *request(const buffer * const b, const char *prompt, const char * const def
 				}
 
 				if (!quoted) {
-					prefix = strrchr(ib.buf, ' ');
-					if (prefix) prefix++;
+					prefix = strchr(ib.buf, ' ');
+					if (prefix) {
+						while (*prefix == ' ') prefix++;
+					}
 					else prefix = ib.buf;
 				}
 
