@@ -1,7 +1,7 @@
 /* Input class, key bindings and upper casing vector definitions.
 
-   Copyright (C) 1993-1998 Sebastiano Vigna 
-   Copyright (C) 1999-2022 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1993-1998 Sebastiano Vigna
+   Copyright (C) 1999-2023 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -23,14 +23,14 @@
 
 const char *input_class_names[INPUT_CLASS_COUNT] = {
 	"ALPHA", "COMMAND", "RETURN", "TAB", "IGNORE", "INVALID"
-}; 
+};
 
 const char *config_source_names[CONFIG_SOURCE_COUNT] = {
 	"Built In",
 	"Global Prefs",      "Global Prefs (terminal)",
 	"User Prefs",        "User Prefs (terminal)",
 	"Current Directory", "Current Directory (terminal)"
-}; 
+};
 
 /* This vector contains all key bindings. Each entry points to a command line
    to be executed when the corresponding keystroke is input. The index
@@ -91,7 +91,7 @@ const char *key_binding[NUM_KEYS] = {
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /* space ! " # $ % & ' ( ) * + , - . /  */
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /*     0 1 2 3 4 5 6 7 8 9 : ; < = > ?  */
 
-	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /* @ A B C D E F G H I J K L M N O  */ 
+	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /* @ A B C D E F G H I J K L M N O  */
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /* P Q R S T U V W X Y Z [ \ ] ^ _  */
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /* ` a b c d e f g h i j k l m n o  */
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,        /* p q r s t u v w x y z { | } ~    */
@@ -99,37 +99,37 @@ const char *key_binding[NUM_KEYS] = {
 
 	/* Control-meta-letter bindings (Ctrl-Alt-X)  128..159 80..9f */
 
-	/* Ctrl-Alt-@ */ NULL, 
-	/* Ctrl-Alt-a */ MOVESOF_ABBREV, 
-	/* Ctrl-Alt-b */ PREVWORD_ABBREV, 
-	/* Ctrl-Alt-c */ MIDDLEVIEW_ABBREV, 
-	/* Ctrl-Alt-d */ NEXTDOC_ABBREV, 
-	/* Ctrl-Alt-e */ MOVEEOF_ABBREV, 
-	/* Ctrl-Alt-f */ NEXTWORD_ABBREV, 
-	/* Ctrl-Alt-g */ GOTOBOOKMARK_ABBREV, 
-	/* Ctrl-Alt-h */ NULL, 
-	/* Ctrl-Alt-i */ AUTOCOMPLETE_ABBREV, 
-	/* Ctrl-Alt-j */ GOTOCOLUMN_ABBREV, 
-	/* Ctrl-Alt-k */ SETBOOKMARK_ABBREV, 
-	/* Ctrl-Alt-l */ TOLOWER_ABBREV, 
-	/* Ctrl-Alt-m */ PLAYONCE_ABBREV, 
-	/* Ctrl-Alt-n */ OPENNEW_ABBREV, 
+	/* Ctrl-Alt-@ */ NULL,
+	/* Ctrl-Alt-a */ MOVESOF_ABBREV,
+	/* Ctrl-Alt-b */ PREVWORD_ABBREV,
+	/* Ctrl-Alt-c */ MIDDLEVIEW_ABBREV,
+	/* Ctrl-Alt-d */ NEXTDOC_ABBREV,
+	/* Ctrl-Alt-e */ MOVEEOF_ABBREV,
+	/* Ctrl-Alt-f */ NEXTWORD_ABBREV,
+	/* Ctrl-Alt-g */ GOTOBOOKMARK_ABBREV,
+	/* Ctrl-Alt-h */ NULL,
+	/* Ctrl-Alt-i */ AUTOCOMPLETE_ABBREV,
+	/* Ctrl-Alt-j */ GOTOCOLUMN_ABBREV,
+	/* Ctrl-Alt-k */ SETBOOKMARK_ABBREV,
+	/* Ctrl-Alt-l */ TOLOWER_ABBREV,
+	/* Ctrl-Alt-m */ PLAYONCE_ABBREV,
+	/* Ctrl-Alt-n */ OPENNEW_ABBREV,
 	/* Ctrl-Alt-o */ OPENCLIP_ABBREV,
 
-	/* Ctrl-Alt-p */ PARAGRAPH_ABBREV, 
-	/* Ctrl-Alt-q */ QUIT_ABBREV, 
-	/* Ctrl-Alt-r */ REDO_ABBREV, 
-	/* Ctrl-Alt-s */ SAVECLIP_ABBREV, 
-	/* Ctrl-Alt-t */ THROUGH_ABBREV, 
-	/* Ctrl-Alt-u */ UNDO_ABBREV, 
-	/* Ctrl-Alt-v */ TOUPPER_ABBREV, 
-	/* Ctrl-Alt-w */ WORDWRAP_ABBREV, 
-	/* Ctrl-Alt-x */ EXIT_ABBREV, 
-	/* Ctrl-Alt-y */ DELETEEOL_ABBREV, 
-	/* Ctrl-Alt-z */ CRLF_ABBREV, 
-	/* Ctrl-Alt-[ */ NULL, 
-	/* Ctrl-Alt-\ */ NULL, 
-	/* Ctrl-Alt-] */ NULL, 
+	/* Ctrl-Alt-p */ PARAGRAPH_ABBREV,
+	/* Ctrl-Alt-q */ QUIT_ABBREV,
+	/* Ctrl-Alt-r */ REDO_ABBREV,
+	/* Ctrl-Alt-s */ SAVECLIP_ABBREV,
+	/* Ctrl-Alt-t */ THROUGH_ABBREV,
+	/* Ctrl-Alt-u */ UNDO_ABBREV,
+	/* Ctrl-Alt-v */ TOUPPER_ABBREV,
+	/* Ctrl-Alt-w */ WORDWRAP_ABBREV,
+	/* Ctrl-Alt-x */ EXIT_ABBREV,
+	/* Ctrl-Alt-y */ DELETEEOL_ABBREV,
+	/* Ctrl-Alt-z */ CRLF_ABBREV,
+	/* Ctrl-Alt-[ */ NULL,
+	/* Ctrl-Alt-\ */ NULL,
+	/* Ctrl-Alt-] */ NULL,
 	/* Ctrl-Alt-^ */ NULL,
 	/* Ctrl-Alt-_ */ NULL,
 
@@ -145,73 +145,73 @@ const char *key_binding[NUM_KEYS] = {
 	/* The following bindings are for the terminfo extended codes (see keycodes.h). */
 
 	/* Cursor movement keys 256..271  100..10F */
-	NULL, 
-	LINEUP_ABBREV, 
-	LINEDOWN_ABBREV, 
-	MOVELEFT_ABBREV, 
-	MOVERIGHT_ABBREV, 
-	PICK(MOVEINCUP_ABBREV, MOVESOL_ABBREV), 
-	PICK(MOVEINCDOWN_ABBREV, MOVEEOL_ABBREV), 
+	NULL,
+	LINEUP_ABBREV,
+	LINEDOWN_ABBREV,
+	MOVELEFT_ABBREV,
+	MOVERIGHT_ABBREV,
+	PICK(MOVEINCUP_ABBREV, MOVESOL_ABBREV),
+	PICK(MOVEINCDOWN_ABBREV, MOVEEOL_ABBREV),
 	PICK(NEXTPAGE_ABBREV, PAGEDOWN_ABBREV),
-	PICK(PREVPAGE_ABBREV, PAGEUP_ABBREV), 
-	LINEDOWN_ABBREV, 
-	LINEUP_ABBREV, 
-	NULL, 
-	NULL, 
-	NULL, 
-	NULL, 
+	PICK(PREVPAGE_ABBREV, PAGEUP_ABBREV),
+	LINEDOWN_ABBREV,
+	LINEUP_ABBREV,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
 	NULL,
 
 	/* Editing keys   272..287   110..11F */
-	DELETEEOL_ABBREV, 
-	NULL, 
-	BACKSPACE_ABBREV, 
-	DELETELINE_ABBREV, 
-	UNDELLINE_ABBREV, 
-	DELETECHAR_ABBREV, 
-	INSERT_ABBREV, 
+	DELETEEOL_ABBREV,
+	NULL,
+	BACKSPACE_ABBREV,
+	DELETELINE_ABBREV,
+	UNDELLINE_ABBREV,
+	DELETECHAR_ABBREV,
+	INSERT_ABBREV,
+	NULL,
+	CLEAR_ABBREV,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
 	NULL,
-	CLEAR_ABBREV, 
-	NULL, 
-	NULL, 
-	NULL, 
-	NULL, 
-	NULL, 
-	NULL, 
 	NULL,
 
 	/* Keypad keys and fake (simulated) menu key  288..303  120..12F */
-	MOVESOF_ABBREV, 
-	PREVPAGE_ABBREV, 
-	TOGGLESEOL_ABBREV, 
-	MOVEEOF_ABBREV, 
-	NEXTPAGE_ABBREV, 
-	EXEC_ABBREV, 
-	NULL, 
+	MOVESOF_ABBREV,
+	PREVPAGE_ABBREV,
+	TOGGLESEOL_ABBREV,
+	MOVEEOF_ABBREV,
+	NEXTPAGE_ABBREV,
+	EXEC_ABBREV,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
 	NULL,
-	NULL, 
-	NULL, 
-	NULL, 
-	NULL, 
-	NULL, 
-	NULL, 
-	NULL, 
 	NULL,
 
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /* 304..319  130..13F */
 
 	/* Function keys   320..335  140..14F  */
-	/*  F0 */ ESCAPE_ABBREV, 
-	/*  F1 */ ESCAPE_ABBREV, 
-	/*  F2 */ NEXTDOC_ABBREV, 
-	/*  F3 */ PREVDOC_ABBREV, 
-	/*  F4 */ SELECTDOC_ABBREV, 
-	/*  F5 */ UNDO_ABBREV, 
-	/*  F6 */ REDO_ABBREV, 
+	/*  F0 */ ESCAPE_ABBREV,
+	/*  F1 */ ESCAPE_ABBREV,
+	/*  F2 */ NEXTDOC_ABBREV,
+	/*  F3 */ PREVDOC_ABBREV,
+	/*  F4 */ SELECTDOC_ABBREV,
+	/*  F5 */ UNDO_ABBREV,
+	/*  F6 */ REDO_ABBREV,
 	/*  F7 */ PREVWORD_ABBREV,
-	/*  F8 */ NEXTWORD_ABBREV, 
-	/*  F9 */ PLAYONCE_ABBREV, 
-	/* F10 */ HELP_ABBREV, 
+	/*  F8 */ NEXTWORD_ABBREV,
+	/*  F9 */ PLAYONCE_ABBREV,
+	/* F10 */ HELP_ABBREV,
 	/* F11 */ NULL,
 	/* F12 */ NULL,
 	/* F13 */ NULL,
@@ -235,7 +235,7 @@ const char *key_binding[NUM_KEYS] = {
    /* F29 */ PICK(NULL, SHIFTSPACELEFT_ABBREV),  /* Ctrl-F5 */
    /* F30 */ PICK(NULL, SHIFTSPACERIGHT_ABBREV), /* Ctrl-F6 */
    /* F31 */ DELETEPREVWORD_ABBREV,             /* Ctrl-F7 */
-   
+
 	/* 352..367  160..16F */
 	/* F32 */ DELETENEXTWORD_ABBREV, /* Ctrl-F8 */
 	/* F33 */ NULL,
@@ -253,119 +253,122 @@ const char *key_binding[NUM_KEYS] = {
 	/* F45 */ NULL,
 	/* F46 */ NULL,
 	/* F47 */ NULL,
-	
+
 	NULL,                  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, /* 368..383  170..17F */
 
 	/* Prefix-simulated META (ESC Ctrl-a)  384..416  180..1A0 */
 
-	/* ESC Ctrl-@ */ NULL, 
-	/* ESC Ctrl-a */ MOVESOF_ABBREV, 
-	/* ESC Ctrl-b */ PREVWORD_ABBREV, 
-	/* ESC Ctrl-c */ MIDDLEVIEW_ABBREV, 
-	/* ESC Ctrl-d */ NEXTDOC_ABBREV, 
-	/* ESC Ctrl-e */ MOVEEOF_ABBREV, 
-	/* ESC Ctrl-f */ NEXTWORD_ABBREV, 
-	/* ESC Ctrl-g */ GOTOBOOKMARK_ABBREV, 
-	/* ESC Ctrl-h */ NULL, 
-	/* ESC Ctrl-i */ AUTOCOMPLETE_ABBREV, 
-	/* ESC Ctrl-j */ GOTOCOLUMN_ABBREV, 
-	/* ESC Ctrl-k */ SETBOOKMARK_ABBREV, 
-	/* ESC Ctrl-l */ TOLOWER_ABBREV, 
-	/* ESC Ctrl-m */ PLAYONCE_ABBREV, 
-	/* ESC Ctrl-n */ OPENNEW_ABBREV, 
+	/* ESC Ctrl-@ */ NULL,
+	/* ESC Ctrl-a */ MOVESOF_ABBREV,
+	/* ESC Ctrl-b */ PREVWORD_ABBREV,
+	/* ESC Ctrl-c */ MIDDLEVIEW_ABBREV,
+	/* ESC Ctrl-d */ NEXTDOC_ABBREV,
+	/* ESC Ctrl-e */ MOVEEOF_ABBREV,
+	/* ESC Ctrl-f */ NEXTWORD_ABBREV,
+	/* ESC Ctrl-g */ GOTOBOOKMARK_ABBREV,
+	/* ESC Ctrl-h */ NULL,
+	/* ESC Ctrl-i */ AUTOCOMPLETE_ABBREV,
+	/* ESC Ctrl-j */ GOTOCOLUMN_ABBREV,
+	/* ESC Ctrl-k */ SETBOOKMARK_ABBREV,
+	/* ESC Ctrl-l */ TOLOWER_ABBREV,
+	/* ESC Ctrl-m */ PLAYONCE_ABBREV,
+	/* ESC Ctrl-n */ OPENNEW_ABBREV,
 	/* ESC Ctrl-o */ OPENCLIP_ABBREV,
 
-	/* ESC Ctrl-p */ PARAGRAPH_ABBREV, 
-	/* ESC Ctrl-q */ QUIT_ABBREV, 
-	/* ESC Ctrl-r */ REDO_ABBREV, 
-	/* ESC Ctrl-s */ SAVECLIP_ABBREV, 
-	/* ESC Ctrl-t */ THROUGH_ABBREV, 
-	/* ESC Ctrl-u */ UNDO_ABBREV, 
-	/* ESC Ctrl-v */ TOUPPER_ABBREV, 
-	/* ESC Ctrl-w */ WORDWRAP_ABBREV, 
-	/* ESC Ctrl-x */ EXIT_ABBREV, 
-	/* ESC Ctrl-y */ DELETEEOL_ABBREV, 
-	/* ESC Ctrl-z */ CRLF_ABBREV, 
-	/* ESC Ctrl-[ */ NULL, 
-	/* ESC Ctrl-\ */ NULL, 
-	/* ESC Ctrl-] */ NULL, 
-	/* ESC Ctrl-^ */ NULL, 
+	/* ESC Ctrl-p */ PARAGRAPH_ABBREV,
+	/* ESC Ctrl-q */ QUIT_ABBREV,
+	/* ESC Ctrl-r */ REDO_ABBREV,
+	/* ESC Ctrl-s */ SAVECLIP_ABBREV,
+	/* ESC Ctrl-t */ THROUGH_ABBREV,
+	/* ESC Ctrl-u */ UNDO_ABBREV,
+	/* ESC Ctrl-v */ TOUPPER_ABBREV,
+	/* ESC Ctrl-w */ WORDWRAP_ABBREV,
+	/* ESC Ctrl-x */ EXIT_ABBREV,
+	/* ESC Ctrl-y */ DELETEEOL_ABBREV,
+	/* ESC Ctrl-z */ CRLF_ABBREV,
+	/* ESC Ctrl-[ */ NULL,
+	/* ESC Ctrl-\ */ NULL,
+	/* ESC Ctrl-] */ NULL,
+	/* ESC Ctrl-^ */ NULL,
 	/* ESC Ctrl-_ */ NULL,
 
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /* 416..431  1A0..1AF */
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /* 432..447  1B0..1BF */
 
 	/* Prefix-simulated META (ESC X)  448..479   1C0..1DF */
-	/* ESC @ */ NULL, 
-	/* ESC A */ MOVESOF_ABBREV, 
-	/* ESC B */ PREVWORD_ABBREV, 
-	/* ESC C */ MIDDLEVIEW_ABBREV, 
-	/* ESC D */ NEXTDOC_ABBREV, 
-	/* ESC E */ MOVEEOF_ABBREV, 
-	/* ESC F */ NEXTWORD_ABBREV, 
-	/* ESC G */ GOTOBOOKMARK_ABBREV, 
-	/* ESC H */ NULL, 
-	/* ESC I */ AUTOCOMPLETE_ABBREV, 
-	/* ESC J */ GOTOCOLUMN_ABBREV, 
-	/* ESC K */ SETBOOKMARK_ABBREV, 
-	/* ESC L */ TOLOWER_ABBREV, 
-	/* ESC M */ PLAYONCE_ABBREV, 
-	/* ESC N */ OPENNEW_ABBREV, 
+	/* ESC @ */ NULL,
+	/* ESC A */ MOVESOF_ABBREV,
+	/* ESC B */ PREVWORD_ABBREV,
+	/* ESC C */ MIDDLEVIEW_ABBREV,
+	/* ESC D */ NEXTDOC_ABBREV,
+	/* ESC E */ MOVEEOF_ABBREV,
+	/* ESC F */ NEXTWORD_ABBREV,
+	/* ESC G */ GOTOBOOKMARK_ABBREV,
+	/* ESC H */ NULL,
+	/* ESC I */ AUTOCOMPLETE_ABBREV,
+	/* ESC J */ GOTOCOLUMN_ABBREV,
+	/* ESC K */ SETBOOKMARK_ABBREV,
+	/* ESC L */ TOLOWER_ABBREV,
+	/* ESC M */ PLAYONCE_ABBREV,
+	/* ESC N */ OPENNEW_ABBREV,
 	/* ESC O */ OPENCLIP_ABBREV,
 
-	/* ESC P */ PARAGRAPH_ABBREV, 
-	/* ESC Q */ QUIT_ABBREV, 
-	/* ESC R */ REDO_ABBREV, 
-	/* ESC S */ SAVECLIP_ABBREV, 
-	/* ESC T */ THROUGH_ABBREV, 
-	/* ESC U */ UNDO_ABBREV, 
-	/* ESC V */ TOUPPER_ABBREV, 
-	/* ESC W */ WORDWRAP_ABBREV, 
-	/* ESC X */ EXIT_ABBREV, 
-	/* ESC Y */ DELETEEOL_ABBREV, 
-	/* ESC Z */ CRLF_ABBREV, 
-	/* ESC [ */ NULL, 
-	/* ESC \ */ NULL, 
-	/* ESC ] */ NULL, 
-	/* ESC ^ */ NULL, 
+	/* ESC P */ PARAGRAPH_ABBREV,
+	/* ESC Q */ QUIT_ABBREV,
+	/* ESC R */ REDO_ABBREV,
+	/* ESC S */ SAVECLIP_ABBREV,
+	/* ESC T */ THROUGH_ABBREV,
+	/* ESC U */ UNDO_ABBREV,
+	/* ESC V */ TOUPPER_ABBREV,
+	/* ESC W */ WORDWRAP_ABBREV,
+	/* ESC X */ EXIT_ABBREV,
+	/* ESC Y */ DELETEEOL_ABBREV,
+	/* ESC Z */ CRLF_ABBREV,
+	/* ESC [ */ NULL,
+	/* ESC \ */ NULL,
+	/* ESC ] */ NULL,
+	/* ESC ^ */ NULL,
 	/* ESC _ */ NULL,
 
 
 	/* Prefix-simulated META (ESC X)  480..511  1E0..1FF */
-	/* ESC @ */ NULL, 
-	/* ESC a */ MOVESOF_ABBREV, 
-	/* ESC b */ PREVWORD_ABBREV, 
-	/* ESC c */ MIDDLEVIEW_ABBREV, 
-	/* ESC d */ NEXTDOC_ABBREV, 
-	/* ESC e */ MOVEEOF_ABBREV, 
-	/* ESC f */ NEXTWORD_ABBREV, 
-	/* ESC g */ GOTOBOOKMARK_ABBREV, 
-	/* ESC h */ NULL, 
-	/* ESC i */ AUTOCOMPLETE_ABBREV, 
-	/* ESC j */ GOTOCOLUMN_ABBREV, 
-	/* ESC k */ SETBOOKMARK_ABBREV, 
-	/* ESC l */ TOLOWER_ABBREV, 
-	/* ESC m */ PLAYONCE_ABBREV, 
-	/* ESC n */ OPENNEW_ABBREV, 
+	/* ESC @ */ NULL,
+	/* ESC a */ MOVESOF_ABBREV,
+	/* ESC b */ PREVWORD_ABBREV,
+	/* ESC c */ MIDDLEVIEW_ABBREV,
+	/* ESC d */ NEXTDOC_ABBREV,
+	/* ESC e */ MOVEEOF_ABBREV,
+	/* ESC f */ NEXTWORD_ABBREV,
+	/* ESC g */ GOTOBOOKMARK_ABBREV,
+	/* ESC h */ NULL,
+	/* ESC i */ AUTOCOMPLETE_ABBREV,
+	/* ESC j */ GOTOCOLUMN_ABBREV,
+	/* ESC k */ SETBOOKMARK_ABBREV,
+	/* ESC l */ TOLOWER_ABBREV,
+	/* ESC m */ PLAYONCE_ABBREV,
+	/* ESC n */ OPENNEW_ABBREV,
 	/* ESC o */ OPENCLIP_ABBREV,
 
-	/* ESC p */ PARAGRAPH_ABBREV, 
-	/* ESC q */ QUIT_ABBREV, 
-	/* ESC r */ REDO_ABBREV, 
-	/* ESC s */ SAVECLIP_ABBREV, 
-	/* ESC t */ THROUGH_ABBREV, 
-	/* ESC u */ UNDO_ABBREV, 
-	/* ESC v */ TOUPPER_ABBREV, 
-	/* ESC w */ WORDWRAP_ABBREV, 
-	/* ESC x */ EXIT_ABBREV, 
-	/* ESC y */ DELETEEOL_ABBREV, 
-	/* ESC z */ CRLF_ABBREV, 
-	/* ESC [ */ NULL, 
-	/* ESC \ */ NULL, 
-	/* ESC ] */ NULL, 
+	/* ESC p */ PARAGRAPH_ABBREV,
+	/* ESC q */ QUIT_ABBREV,
+	/* ESC r */ REDO_ABBREV,
+	/* ESC s */ SAVECLIP_ABBREV,
+	/* ESC t */ THROUGH_ABBREV,
+	/* ESC u */ UNDO_ABBREV,
+	/* ESC v */ TOUPPER_ABBREV,
+	/* ESC w */ WORDWRAP_ABBREV,
+	/* ESC x */ EXIT_ABBREV,
+	/* ESC y */ DELETEEOL_ABBREV,
+	/* ESC z */ CRLF_ABBREV,
+	/* ESC [ */ NULL,
+	/* ESC \ */ NULL,
+	/* ESC ] */ NULL,
 	/* ESC ^ */ NULL,
-	/* ESC _ */ PICK(NULL, AUTOINDENT_ABBREV)
+	/* ESC _ */ PICK(NULL, AUTOINDENT_ABBREV),
 
+   /* Two fake ones for bracketed paste        200..201 */
+   /*  n/a  */ NULL,  /* NE_KEY_BPASTE_BEGIN */
+   /*  n/a  */ NULL,  /* NE_KEY_BPASTE_END */
 };
 
 
@@ -550,42 +553,42 @@ const char *key_stroke[NUM_KEYS] = {
 	"^B",
 	"^C",
 	"^D",
-	"^E",      
-	"^F",      
-	"^G",      
+	"^E",
+	"^F",
+	"^G",
 
-	"^H",      
-	"^I",      
-	"^J",      
-	"^K",      
-	"^L",      
-	"^M",      
-	"^N", 
-	"^O",      
+	"^H",
+	"^I",
+	"^J",
+	"^K",
+	"^L",
+	"^M",
+	"^N",
+	"^O",
 
-	"^P", 
-	"^Q", 
-	"^R", 
-	"^S", 
-	"^T", 
-	"^U", 
-	"^V", 
-	"^W", 
+	"^P",
+	"^Q",
+	"^R",
+	"^S",
+	"^T",
+	"^U",
+	"^V",
+	"^W",
 
-	"^X", 
-	"^Y", 
-	"^Z", 
-	"^[", 
-	"^\\", 
-	"^]", 
-	"^^", 
-	"^_", 
+	"^X",
+	"^Y",
+	"^Z",
+	"^[",
+	"^\\",
+	"^]",
+	"^^",
+	"^_",
 
 	/* These keys map to strings, not commands. [32..127)  [20..7f) */
 	" ", "!", "\"","#", "$", "%", "&", "'", "(", ")", "*", "+", ",", "-", ".", "/",   /* space ! " # $ % & ' ( ) * + , - . /  */
 	"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", ":", ";", "<", "=", ">", "?",   /*     0 1 2 3 4 5 6 7 8 9 : ; < = > ?  */
 
-	"@", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O",   /* @ A B C D E F G H I J K L M N O  */ 
+	"@", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O",   /* @ A B C D E F G H I J K L M N O  */
 	"P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "[", "\\","]", "^", "_",   /* P Q R S T U V W X Y Z [ \ ] ^ _  */
 	"`", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o",   /* ` a b c d e f g h i j k l m n o  */
 	"p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "{", "|", "}", "~",        /* p q r s t u v w x y z { | } ~    */
@@ -639,7 +642,7 @@ const char *key_stroke[NUM_KEYS] = {
 	/* The following bindings are for the terminfo extended codes (see keycodes.h). */
 
 	/* Cursor movement keys [256..272)  [100..110) */
-	"", 
+	"",
 	"Up",
 	"Down",
 	"Left",
@@ -650,15 +653,15 @@ const char *key_stroke[NUM_KEYS] = {
 	"PgUp",
 	"LnDn",
 	"LnUp",
-	"", 
-	"", 
-	"", 
-	"", 
+	"",
+	"",
+	"",
+	"",
 	"",
 
 	/* Editing keys   [272..288)   [110..120) */
 	"Del",
-	"", 
+	"",
 	"BackSp",
 	"DelLn",
 	"UndelLn",
@@ -666,12 +669,12 @@ const char *key_stroke[NUM_KEYS] = {
 	"Ins",
 	"",
 	"Clear",
-	"", 
-	"", 
-	"", 
-	"", 
-	"", 
-	"", 
+	"",
+	"",
+	"",
+	"",
+	"",
+	"",
 	"",
 
 	/* Keypad keys and fake (simulated) menu key  [288..304)  [120..130) */
@@ -681,15 +684,15 @@ const char *key_stroke[NUM_KEYS] = {
 	"EOF",
 	"NxtPg",
 	"Exec",
-	"", 
 	"",
-	"", 
-	"", 
-	"", 
-	"", 
-	"", 
-	"", 
-	"", 
+	"",
+	"",
+	"",
+	"",
+	"",
+	"",
+	"",
+	"",
 	"",
 
 	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* [304..320)  [130..140) */
@@ -858,4 +861,7 @@ const char *key_stroke[NUM_KEYS] = {
 	"ESC-~",
 	"ESC-DEL",
 
+	/* Bracketed Paste Begin and End [512, 513] [200, 201] */
+	"*BPB*",
+	"*BPE*",
 };
