@@ -1,7 +1,7 @@
 /* main(), global initialization and global buffer functions.
 
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2022 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2023 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -36,7 +36,7 @@
 
 char *NO_WARRANTY_msg[] = {   PROGRAM_NAME " " VERSION ".",
 										"Copyright (C) 1993-1998 Sebastiano Vigna",
-										"Copyright (C) 1999-2022 Todd M. Lewis and Sebastiano Vigna",
+										"Copyright (C) 1999-2023 Todd M. Lewis and Sebastiano Vigna",
 										"",
 										"This program is free software; you can redistribute it and/or modify it under",
 										"the terms of the GNU General Public License as published by the Free Software",
@@ -74,6 +74,7 @@ char ARG_HELP[] = ABOUT_MSG "\n"
 						"--no-ansi     do not use built-in ANSI control sequences. [--noansi]\n"
 						"--no-config   do not read configuration files. [--noconfig]\n"
 						"--no-syntax   disable syntax-highlighting support.\n"
+						"--no-bpaste   disable bracketed paste support. [--nobpaste]\n"
 						"--prefs EXT   set autoprefs for the provided extension before loading the first file.\n"
 						"--keys FILE   use this file for keyboard configuration.\n"
 						"--menus FILE  use this file for menu configuration.\n"
@@ -104,9 +105,11 @@ bool fast_gui;
 bool status_bar = true;
 bool interactive_mode;
 bool verbose_macros = true;
+bool bracketed_paste = true;
 /* end of global prefs */
 
 buffer *cur_buffer;
+unsigned long buffer_actuations = 0;
 int turbo;
 bool do_syntax = true;
 
@@ -132,7 +135,11 @@ buffer *new_buffer(void) {
 }
 
 bool delete_buffer(void) {
-	buffer *b = (buffer *)cur_buffer->b_node.next;
+	buffer *b;
+	if (is_first(cur_buffer)) b = (buffer *)cur_buffer->b_node.next;
+	else if (is_last(cur_buffer)) b = (buffer *)cur_buffer->b_node.prev;
+	else if (((buffer *)cur_buffer->b_node.next)->act > ((buffer *)cur_buffer->b_node.prev)->act) b = (buffer *)cur_buffer->b_node.next;
+	else b = (buffer *)cur_buffer->b_node.prev;
 
 	rem(&cur_buffer->b_node);
 	free_buffer(cur_buffer);
@@ -207,18 +214,23 @@ int main(int argc, char **argv) {
 
 		if (argv[i][0] == '-' && argv[i][1] == '-') {
 			if (!argv[i][2]) i++; /* You can use "--" to force the next token to be a filename */
-			else if (!strcmp(&argv[i][2], "noconfig") || !strcmp(&argv[i][2], "no-config")) {
+			else if (!strcmp(&argv[i][2], "no-config") || !strcmp(&argv[i][2], "noconfig")) {
 				no_config = true;
 				skiplist[i] = 1; /* argv[i] = NULL; */
 			}
-			else if (!strcmp(&argv[i][2], "noansi") || !strcmp(&argv[i][2], "no-ansi")) {
+			else if (!strcmp(&argv[i][2], "no-ansi") || !strcmp(&argv[i][2], "noansi")) {
 				ansi = false;
 				skiplist[i] = 1; /* argv[i] = NULL; */
 			}
-			else if (!strcmp(&argv[i][2], "no-syntax")) {
+			else if (!strcmp(&argv[i][2], "no-syntax") || !strcmp(&argv[i][2], "nosyntax")) {
 				do_syntax = false;
 				skiplist[i] = 1; /* argv[i] = NULL; */
 			}
+			else if (!strcmp(&argv[i][2], "no-bpaste") || !strcmp(&argv[i][2], "nobpaste")) {
+				turn_off_bracketed_paste();
+				bracketed_paste = false;
+				skiplist[i] = 1; /* argv[i] = NULL; */
+			}
 			else if (!strcmp(&argv[i][2], "prefs")) {
 				if (i < argc-1) {
 					startup_prefs_name = argv[i+1];
@@ -458,7 +470,7 @@ int main(int argc, char **argv) {
 		free(skiplist);
 
 		/* This call makes current the first specified file. It is called
-		   only if more than one buffer exist. */
+		   only if more than one buffer exists. */
 
 		if (get_nth_buffer(1)) do_action(cur_buffer, NEXTDOC_A, -1, NULL);
 
@@ -531,6 +543,8 @@ int main(int argc, char **argv) {
 		case COMMAND:
 			if (c < 0) c = -c - 1;
 			if (key_binding[c]) print_error(execute_command_line(cur_buffer, key_binding[c]));
+			else if (c == NE_KEY_BPASTE_BEGIN && bracketed_paste) bracketed_paste_begin(cur_buffer);
+			else if (c == NE_KEY_BPASTE_END   && bracketed_paste) bracketed_paste_end(cur_buffer);
 			break;
 
 		default:
