@@ -1,7 +1,7 @@
 /* Main typedefs and defines.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna
-	Copyright (C) 1999-2022 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2023 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -116,9 +116,10 @@ extern const char *config_source_names[];
 
 #define UNNAMED_NAME       "<unnamed>"
 
-/* The number of key codes, including ISO-8859-1 plus 256 extra codes. */
+/* The number of key codes, including ISO-8859-1 plus 256 extra codes,
+   plus two more reserved for bracketed paste begin and end. */
 
-#define NUM_KEYS           (256+256)
+#define NUM_KEYS           (256+256+2)
 
 /* A key code that has class INVALID. */
 
@@ -149,7 +150,7 @@ typedef enum {
 	ENC_ASCII = 0, ENC_UTF8, ENC_8_BIT
 } encoding_type;
 
-/* Bookmark designations. User get 0 through 9 plus the AUTO_BOOKMARK '-'. */
+/* Bookmark designations. Users get 0 through 9 plus the AUTO_BOOKMARK '-'. */
 
 enum {
 	MAX_USER_BOOKMARK = 9,
@@ -163,7 +164,7 @@ enum {
 enum {
 	COMPLETE_NONE = 0,
 	COMPLETE_FILE,
-	COMPLETE_CMD_FILE,  /* Unimplemented ??? */
+	COMPLETE_CMD,  /* Unimplemented ??? */
 	COMPLETE_SYNTAX
 };
 
@@ -207,7 +208,8 @@ typedef struct {
 	node *tail_pred;
 } list;
 
-
+#define is_first(n) (((node *)n)->prev->prev == NULL)
+#define is_last(n)  (((node *)n)->next->next == NULL)
 
 /* All the structures that follow have an assertion macro which checks for
    partial internal consistency of the structure. These macros are extensively
@@ -483,7 +485,11 @@ typedef struct {
 	char *find_string;
 	char *replace_string;
 	char *command_line;
+	char *bpaste_macro_before; /* name of macro to run before processing bracketed pastes */
+	char *bpaste_macro_after;  /* name of macro to run after processing bracketed pastes */
+	int   bpaste_support;      /* 0=not supported; 1=supported w/ defaults; 2=supported with before and after macros */
 	unsigned long mtime;      /* mod time of on-disk file when it was last loaded/saved, or 0 */
+	unsigned long act;        /* increasing indicator of recently active relative to other buffers */
 	int64_t win_x, win_y;     /* line and pos of upper left-most visible character. */
 	int cur_x, cur_y;         /* position of cursor within the window */
 	int64_t wanted_x;         /* desired x position modulo short lines, tabs, etc. Valid only if x_wanted is true. */
@@ -555,7 +561,7 @@ typedef struct {
 	assert((b)->cur_pos == -1 || calc_width((b)->cur_line_desc, (b)->cur_pos, (b)->opt.tab_size, (b)->encoding) == (b)->win_x + (b)->cur_x);\
 	assert((b)->cur_pos == -1 || calc_virt_pos((b)->cur_line_desc, (b)->win_x + (b)->cur_x, (b)->opt.tab_size, (b)->encoding) == (b)->cur_pos);\
 	assert((b)->cur_pos == -1 || (b)->cur_pos > (b)->cur_line_desc->line_len || calc_char_len((b)->cur_line_desc, (b)->cur_pos, (b)->encoding) == (b)->cur_char);\
-	assert((b)->cur_pos < 0 || (b)->cur_pos >= (b)->cur_line_desc->line_len || (b)->encoding != ENC_UTF8 || utf8len((b)->cur_line_desc->line[(b)->cur_pos] > 0));\
+	assert((b)->cur_pos < 0 || (b)->cur_pos >= (b)->cur_line_desc->line_len || (b)->encoding != ENC_UTF8 || utf8len((b)->cur_line_desc->line[(b)->cur_pos]) > 0);\
 	assert_undo_buffer(&(b)->undo);\
 }}
 
@@ -582,6 +588,8 @@ extern config_source key_binding_source[NUM_KEYS];
 
 extern buffer *cur_buffer;
 
+extern unsigned long buffer_actuations;
+
 /* These are the global lists. */
 
 extern list buffers, clips;
