@@ -1,7 +1,7 @@
 /* Miscellaneous support functions.
 
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2022 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2023 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -142,7 +142,7 @@ char *relative_file_path(const char *aa, const char *b) {
 	if (!a) return NULL;
 	normalize_path(a);
 
-	int match = max_prefix(a, b);
+	int match = max_prefix(a, ENC_8_BIT, b, ENC_8_BIT);
 	if (a[0] != '/' || b[0] != '/') {
 		if (a) free(a);
 		return NULL;
@@ -415,10 +415,13 @@ bool same_str(const char *p, const char *q) {
 
 /* Computes the length of the maximal common prefix of s and t. */
 
-int max_prefix(const char * const s, const char * const t) {
-	int i;
-	for(i = 0; s[i] && t[i] && s[i] == t[i]; i++);
-	return i;
+int max_prefix(const char * const s, encoding_type s_enc, const char * const t, encoding_type t_enc) {
+	int s_pos = 0, t_pos = 0;
+	while (s[s_pos] && t[t_pos] && get_char(&s[s_pos], s_enc) == get_char(&t[t_pos], t_enc)) {
+		s_pos = next_pos(s, s_pos, s_enc);
+		t_pos = next_pos(t, t_pos, t_enc);
+	}
+	return s_pos;
 }
 
 
@@ -604,7 +607,7 @@ bool copy_file(const int in, const int out, size_t size) {
    true on success, false on error. */
 
 bool zero_file(const int out, size_t size) {
-	char buffer[8192] = {};
+	char buffer[8192] = {0};
 
 	while(size) {
 		const ssize_t to_do = min(size, sizeof buffer);
@@ -729,3 +732,23 @@ const char *cur_bookmarks_string(const buffer *b) {
 	}
 	return str;
 }
+
+#define MAX_MESSAGE_SIZE 1024
+static char str[MAX_MESSAGE_SIZE+1];
+const char *cur_bracketed_paste_value(const buffer *b) {
+	char *s = str;
+	memset(str, 0, MAX_MESSAGE_SIZE);
+	if (b->bpaste_support < 1) *s = '0';
+	else if (b->bpaste_support == 1) *s = '1';
+	else strncat(strncat(strncpy(str, b->bpaste_macro_before, MAX_MESSAGE_SIZE)," ",MAX_MESSAGE_SIZE), b->bpaste_macro_after, MAX_MESSAGE_SIZE);
+	return str;
+}
+
+const char *cur_bracketed_paste_string(const buffer *b) {
+	memset(str, 0, MAX_MESSAGE_SIZE);
+	strncpy(str, "BracketedPaste ", MAX_MESSAGE_SIZE);
+	if (b->bpaste_support < 1) strncat(str, "disabled", MAX_MESSAGE_SIZE);
+	else if (b->bpaste_support == 1) strncat(str, "enabled with internal defaults", MAX_MESSAGE_SIZE);
+	else if (b->bpaste_support == 2) strncat(strncat(strncat(strncat(str, "macros: ", MAX_MESSAGE_SIZE), b->bpaste_macro_before, MAX_MESSAGE_SIZE), ", ", MAX_MESSAGE_SIZE), b->bpaste_macro_after, MAX_MESSAGE_SIZE);
+	return str;
+}
