@@ -3,7 +3,7 @@
 
    Copyright (C) 1985, 1986, 1987 Free Software Foundation, Inc.
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2022 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2023 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -230,7 +230,7 @@ static bool standout_mode;         /* True when in standout mode. */
 static bool standout_wanted;       /* True if we should be writing in standout mode. */
 static bool underline_mode;        /* True when in underline mode. */
 static bool underline_wanted;      /* True if we should be writing in underline mode. */
-
+static bool bpaste_support_enabled;
 /* Size of window specified by higher levels. This is the number of lines,
    starting from top of screen, to participate in ins/del line operations.
    Effectively it excludes the bottom lines - specified_window_size lines from
@@ -562,6 +562,15 @@ static void turn_off_insert (void) {
 	insert_mode = false;
 }
 
+void turn_on_bracketed_paste(void) {
+	if (bpaste_support_enabled) OUTPUT1_IF(BPASTE_ENABLE_SEQ);
+	bpaste_support_enabled = true;
+}
+
+void turn_off_bracketed_paste(void) {
+	if (bpaste_support_enabled) OUTPUT1_IF(BPASTE_DISABLE_SEQ);
+	bpaste_support_enabled = false;
+}
 
 
 /* Prepares the terminal for interactive I/O. It
@@ -581,6 +590,7 @@ void set_terminal_modes(void) {
 	OUTPUT1_IF(ne_exit_underline_mode);
 	OUTPUT1_IF(ne_enter_ca_mode);
 	OUTPUT1_IF(ne_keypad_xmit);
+	if (bracketed_paste) OUTPUT1_IF(BPASTE_ENABLE_SEQ);
 
 	if (ne_has_meta_key) OUTPUT1_IF(ne_meta_on);
 		turn_off_standout();
@@ -593,6 +603,7 @@ void set_terminal_modes(void) {
 
 void reset_terminal_modes (void) {
 
+	if (bracketed_paste) OUTPUT1_IF(BPASTE_DISABLE_SEQ);
 	OUTPUT1_IF(ne_exit_attribute_mode);
 	OUTPUT1_IF(ne_exit_alt_charset_mode);
 	turn_off_standout();
