@@ -1,7 +1,7 @@
 /* Function prototypes for term.c
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2022 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2023 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -24,6 +24,8 @@
 int output_width(int c);
 void ring_bell(void);
 void do_flash(void);
+void turn_off_bracketed_paste(void);
+void turn_on_bracketed_paste(void);
 void set_terminal_modes(void);
 void reset_terminal_modes(void);
 void set_terminal_window(int size);
