@@ -1,7 +1,7 @@
 /* extern's of flags describing terminal's characteristics.
 
    Copyright (C) 1993-1998 Sebastiano Vigna
-   Copyright (C) 1999-2022 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1999-2023 Todd M. Lewis and Sebastiano Vigna
 
    This file is part of ne, the nice editor.
 
@@ -199,3 +199,27 @@ extern bool ne_transparent_underline;
 extern bool io_utf8;
 
 extern uint32_t curr_attr;
+
+/* bpaste_support=-1 - disable at command line. Do no bpaste operations.
+
+   bpaste_support=1 - the default - sets "AutoIndent 0" and "AtomicUndo 1".
+
+   bpaste_support=0 disables bracketed paste support.
+
+   bpaste_support=2 enables bracketed paste support, and bpaste_macro_before
+     and bpaste_macro_after point to names of macros to execute before and
+     after processing bracketed paste input.
+
+   If bpaste_support > 0, then ne will emit the BPASTE_ENABLE_SEQ escape
+   sequence whenever the terminal is set, and emit BPASTE_DISABLE_SEQ when
+   relinquishing control of the terminal. */
+
+#define BPASTE_DEFAULT      1
+#define BPASTE_ENABLE_SEQ   "\x1b[?2004h"
+#define BPASTE_DISABLE_SEQ  "\x1b[?2004l"
+#define BPASTE_BEGIN_SEQ    "\x1b[200~"
+#define BPASTE_END_SEQ      "\x1b[201~"
+extern bool  bracketed_paste;
+extern char *bpaste_macro_before;
+extern char *bpaste_macro_after;
+
